angular.module('app.controllers')
  .controller('gameCtrl', function ($scope, $state, $ionicPopup, $timeout, $stateParams, $ionicPlatform, $translate, ItemDB, Profile, Utils, Gameservice, ACHIEVEMENTS, Store, Analytics, THEMES, GameHelper) {

    // Fab init
    $scope.fab = false;
    $scope.closeFab = function () {
      $timeout(function () {
        $scope.$apply(function () {
          if ($scope.fab) $scope.fab = false;
        });
      }, 0);
    };

    // Check for plugin support
    var hasPluginSupport = true;
    try {
      window.plugins.insomnia;
    } catch (e) {
      hasPluginSupport = false;
    }

    // Variables init
    var ignorePause = false;
    var saveGameFunction, undoFunction, loadFunction, onPause, onResume, gameContainer;

    // Function button methods
    $scope.clearAll = function () {
      $translate(['CLEAR', 'CANCEL', 'RESET_GAME']).then(function (translations) {
        $ionicPopup.show({
          template: translations.RESET_GAME,
          buttons: [
            { text: translations.CANCEL },
            {
              text: translations.CLEAR,
              type: 'button-positive',
              onTap: function () {
                gameContainer.clearAll();
                $scope.fab = false;
              }
            }
          ]
        });
      });
    };

    $scope.saveGameSlot = function () {
      if (Profile.getHasFunction('function_save_load')) {
        ItemDB.insertOrUpdate(gameContainer.saveGameSlot(0));
        $scope.fab = false;
        Utils.showTranslatedMessage('SUCCESSFULLY_SAVED');
      } else {
        ignorePause = true;
        Store.purchaseFunction('function_save_load', function () {
          $scope.saveGameSlot();
          ignorePause = false;
        }, function () {
          ignorePause = false;
        });
      }
    };

    $scope.restoreGameSlot = function () {
      if (Profile.getHasFunction('function_save_load')) {
        gameContainer.restoreGameSlot(0, function (gameObject) {
          ItemDB.insertOrUpdate(gameObject);
          $scope.fab = false;
        }, function () {
          Utils.showTranslatedMessage('NO_SAVEGAME_AVAILABLE');
        });
      } else {
        ignorePause = true;
        Store.purchaseFunction('function_save_load', function () {
          $scope.restoreGameSlot();
          ignorePause = false;
        }, function () {
          ignorePause = false;
        });
      }
    };

    $scope.undo = function () {
      gameContainer.undo(null, function () {
        Utils.showTranslatedMessage('NO_MORE_STEPS');
        $scope.fab = false;
      });
    };

    $scope.dialogUseHint = function () {
      GameHelper.dialogUseHint(gameContainer.hintApproved);
    };

    /*
     *
     */
    $scope.finishGame = function () {
      $scope.currentGame.played = true;
      Analytics.trackGameResult($scope.currentGame.id, $scope.currentGame.isCorrect);
      Analytics.trackGameTime($scope.currentGame.id, $scope.currentGame.duration);

      if ($scope.currentGame.isCorrect) {
        Profile.removeResumableGameId();
        if ($scope.currentGame.isCampaign) {
          $scope.currentGame.score = Profile.calculatePoints($scope.currentGame.duration, true);
          Profile.increaseLevel();
        } else {
          $scope.currentGame.score = Profile.calculatePoints($scope.currentGame.duration, true, $scope.currentGame.category);
        }
        $scope.currentGame.state = 3;
      } else {
        $scope.currentGame.state = 2;

        $timeout(function () {
          $scope.$apply(function () {
            $scope.currentGame.isFinished = false;
            $scope.currentGame.played = false;
            $scope.currentGame.isCorrect = false;
            delete $scope.currentGame.savedGame;
            ItemDB.insertOrUpdate($scope.currentGame);
          });
        }, 0);

      }
      ItemDB.insertOrUpdate($scope.currentGame);

      // TODO Define special achievements
      if ($scope.currentGame.duration < 60) {
        Gameservice.unlockAchievement(ACHIEVEMENTS[5].id);
      }

      if ($scope.currentGame.isCampaign) {
        Profile.addCampaignTime($scope.currentGame.duration);
        Utils.navigateTo('result-campaign', { id: $scope.currentGame.id });
      } else {
        if ($scope.currentGame.id == "TUTORIAL") {
          Utils.navigateTo('result-tutorial', { id: $scope.currentGame.id });
        } else if ($scope.currentGame.id.startsWith("PROMO")) {
          Utils.navigateTo('result-promo', { id: $scope.currentGame.id });
        } else {
          Utils.navigateTo('result-freegame', { id: $scope.currentGame.id });
        }
      }
    };

    // Use lifecycle methods for insomnia (avoid screen to turn off when not used)
    $scope.$on('$ionicView.loaded', function () {
      $ionicPlatform.ready(function () {
        onPause = $ionicPlatform.on('pause', function () {
          if (saveGameFunction && !ignorePause) {
            saveGameFunction();
            Utils.navigateTo('app', {}, true);
          }
          if (hasPluginSupport) window.plugins.insomnia.allowSleepAgain();
        });

        onResume = $ionicPlatform.on('resume', function () {
          if (hasPluginSupport) window.plugins.insomnia.keepAwake()
        });
      });
    });

    $scope.$on('$ionicView.unloaded', function () {
      if (onPause) {
        onPause();
      }
      if (onResume) {
        onResume();
      }
      if (hasPluginSupport) window.plugins.insomnia.allowSleepAgain();
    });

    $scope.$on('$ionicView.beforeLeave', function () {
      if (saveGameFunction) {
        saveGameFunction();
      }
      if (hasPluginSupport) window.plugins.insomnia.allowSleepAgain();
    });



    $scope.$on('$ionicView.beforeEnter', function () {
      if (hasPluginSupport) window.plugins.insomnia.keepAwake();

      // Calculate ratios for iOS special behavior
      $scope.ratios = Utils.calculateRatios();

      // Load game from path and mark it as running
      $scope.currentGame = ItemDB.selectById($stateParams.id);
      $scope.currentGame.themehex = THEMES[$scope.currentGame.theme];
      $scope.currentGame.state = 1;
      ItemDB.insertOrUpdate($scope.currentGame);

      $ionicPlatform.registerBackButtonAction(function () {
        if ($scope.currentGame.isCampaign) {
          $scope.navigateTo('app');
        } else {
          $scope.navigateTo('free-game');
        }
      }, 600);

      if (ionic.Platform.isAndroid()) {
        StatusBar.backgroundColorByHexString(THEMES[$scope.currentGame.theme]);
      }
    });


    $scope.goBack = function () {
      if (saveGameFunction) {
        saveGameFunction();
      }
      if ($scope.currentGame.isCampaign || $stateParams.id == 'TUTORIAL' || $stateParams.id.startsWith('PROMO')) {
        Utils.navigateTo('app');
      } else {
        Utils.navigateTo('free-game');
      }
    };

    $scope.gameListener = function (gameObject) {
      $timeout(function () {
        $scope.$apply(function () {
          $scope.currentGame.isFinished = gameObject;
        });
      }, 0);
    };

    var hasNotch = function () {
      var proceed = false;
      var div = document.createElement('div');
      if (CSS.supports('padding-bottom: env(safe-area-inset-bottom)')) {
        div.style.paddingBottom = 'env(safe-area-inset-bottom)';
        proceed = true;
      } else if (CSS.supports('padding-bottom: constant(safe-area-inset-bottom)')) {
        div.style.paddingBottom = 'constant(safe-area-inset-bottom)';
        proceed = true;
      }
      if (proceed) {
        document.body.appendChild(div);
        var calculatedPadding = parseInt(window.getComputedStyle(div).paddingBottom);
        document.body.removeChild(div);
        if (calculatedPadding > 0) {
          return true;
        }
      }
      return false;
    };

    $scope.iframeLoadedCallBack = function () {
      $translate(['HINTS', Profile.getRankLabel(), $scope.currentGame.category.toUpperCase(), 'GAME_OF_THE_WEEK']).then(function (translations) {

        var configuration = {
          pixelRatio: $scope.ratios.innerRatio,
          isTablet: window.isTablet || hasNotch(),
          showRulesDialogAfter: 60000,
          rootPath: '../../../js/branding/'
        };

        var translations = {
          rank: translations[Profile.getRankLabel()],
          hintsLabel: translations.HINTS,
          difficulty: $scope.currentGame.id.startsWith('PROMO') ? translations['GAME_OF_THE_WEEK'] : translations[$scope.currentGame.category.toUpperCase()] + ' ' + $stateParams.publicName,
          playerRank: Profile.getLevelInRank(),
          playerRankLabel: Profile.getRankLabel(),
          hints: Profile.getProfile().hints
        };


        var callbacks = {
          dialogNonRemovable: GameHelper.dialogNonRemovable,
          dialogShowRules: GameHelper.dialogShowRules,
          dialogUseHint: GameHelper.dialogUseHint,
          saveGameItem: GameHelper.saveGameItem,
          gameListener: $scope.gameListener,
          closeFab: $scope.closeFab
        };

        gameContainer = document.getElementById('iframe').contentWindow.play($scope.currentGame, configuration, translations, callbacks);
        saveGameFunction = gameContainer.saveGame;

      });
    };
  });
