var preloadOpacity = 0.75;
var blockTypes = {
  X: 'X',
  O: 'O'
};

var style = {
  boardBackgroundColor : 0xFFFFFF,
  boardLineColor : 0x666666,
  shadeBorderSize : 20,
  gameBorder: 4,
  outerGameBorder: 6,
  boardTextStyleWhiteX : {fontSize: 14, font: "RobotoDraft Regular", fill: "#000000"},
  boardTextStyleWhiteXTablet : {fontSize: 30, font: "RobotoDraft Regular", fill: "#000000"},
  boardTextStyleWhiteO : {fontSize: 14, font: "RobotoDraft Regular", fill: "#03a9f4"},
  boardTextStyleWhiteOTablet : {fontSize: 30, font: "RobotoDraft Regular", fill: "#03a9f4"},
  timerStyle : {fontSize: 12, font: "RobotoDraft Medium", fill: 'rgba(0,0,0,0.87)'},
  rankStyle : {fontSize: 12, font: "Arial", fill: 'rgba(0,0,0,0.87)'},
  hintsStyle : {fontSize: 18, font: "Arial", fill: 'rgba(0,0,0,0.87)'},
  hintsLabelStyle : {fontSize: 8, font: "Arial", fill: 'rgba(0,0,0,0.87)'}
};

var spacing = {
  offsetLeftMobile : 0,
  offsetTopMobile : 75,
  offsetHeaderMobile : 20,
  offsetLeftTablet : 0,
  offsetTopTablet : 120,
  offsetHeaderTablet : 55,
  spacingHeader : 14
};


var play = function (initialGameObject, configuration, translations, callbacks) {
  game = new Phaser.Game(window.innerWidth * configuration.pixelRatio, window.innerHeight * configuration.pixelRatio, Phaser.CANVAS, 'gameArea');

  gameObject = initialGameObject;
  var board, boardSolution, boardRectangles, columnCounter, rowCounter, rowLabelsX, rowLabelsO, columnLabelsX, columnLabelsO, field, lines, graphics, tileSize, timerText, rankText, hintsText, hintsLabel, saveCpu, isFinished;
  var commandsStack = [];
  var startTime = new Date();
  var lastAction = new Date().getTime();
  var remindedOfRules = false;

  var playGame = function (game) {
  };
  playGame.prototype = {
    preload: function () {

      // Set global background color
      game.stage.backgroundColor = gameObject.themehex;

      // Load game resources
      game.load.image('X', configuration.rootPath + 'img/X.png');
      game.load.image('O', configuration.rootPath + 'img/O.png');
      game.load.image('norank', configuration.rootPath + 'img/ranks/ic_brightness_5_white_48dp.png');
      game.load.image('rank', configuration.rootPath + 'img/ranks/ic_brightness_7_white_48dp.png');
      game.load.image('background', configuration.rootPath + 'img/themes/' + gameObject.theme + '.png');

      // Load game data
      game.load.text('csv', configuration.rootPath + 'csv/' + gameObject.path + gameObject.id + '.csv');

      // By default if the browser tab loses focus the game will pause. You can stop that behaviour by setting this property to true.
      game.stage.disableVisibilityChange = true;

      // A scale mode that shows the entire game while maintaining proportions - see scaleMode.
      game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
      game.scale.refresh();

      // Make use of advanced timing (use by the savecpu plugin)
      game.time.advancedTiming = true;


    },
    create: function () {

      // Configure save cpu plugin
      saveCpu = game.plugins.add(Phaser.Plugin.SaveCPU);
      saveCpu.renderOnFPS = 1;
      saveCpu.renderOnPointerChange = true;

      // Initialize gameboard
      parseCSV();
      calculateSizes();
      drawField();
      initTimer();
      initRank();
      initHints();
      loadField();
      loadSolution();
      loadSaveGame();

      // Register touch listener
      game.input.onDown.add(onTap, this);

      // Register idle listener
      setInterval(checkIdle, configuration.showRulesDialogAfter - 1000);
    },

    update: function () {
      updateTime();
    }

  };

  /*
   * Calculates all necessary sizes depending on the device height / width / pixel density
   */
  function calculateSizes() {
    spacing.offsetLeftMobile *= configuration.pixelRatio;
    spacing.offsetTopMobile *= configuration.pixelRatio;
    spacing.offsetHeaderMobile *= configuration.pixelRatio;
    spacing.spacingHeader *= configuration.pixelRatio;

    style.shadeBorderSize *= configuration.pixelRatio;
    style.timerStyle = {font: style.timerStyle.fontSize * configuration.pixelRatio + "px " + style.timerStyle.font, fill: style.timerStyle.fill};
    style.rankStyle = {font: style.rankStyle.fontSize * configuration.pixelRatio + "px " + style.rankStyle.font, fill: style.rankStyle.fill};
    style.hintsStyle = {font: style.hintsStyle.fontSize * configuration.pixelRatio + "px " + style.hintsStyle.font, fill: style.hintsStyle.fill};
    style.hintsLabelStyle = {font: style.hintsLabelStyle.fontSize * configuration.pixelRatio + "px " + style.hintsLabelStyle.font, fill: style.hintsLabelStyle.fill};

    var size = game.world.width;
    // Switch size height <-> width for landscape devices
    if (game.world.width > game.world.height) {
      size = game.world.height;
    }

    tileSize = Math.round((size - spacing.offsetLeftMobile) / (gameObject.fieldSize + 3));
    spacing.offsetLeftMobile = tileSize;

    // Recalculate some variables when on tablet
    if (configuration.isTablet) {
      if (gameObject.fieldSizeY != 14) {
        spacing.offsetTopMobile = spacing.offsetTopTablet * configuration.pixelRatio;
        spacing.offsetHeaderMobile = spacing.offsetHeaderTablet * configuration.pixelRatio;
      }
      style.hintsLabelStyle = {font: 14 * configuration.pixelRatio + "px Arial", fill: 'rgba(0,0,0,0.87)'};
      style.boardTextStyleWhiteX = {font: style.boardTextStyleWhiteXTablet.fontSize * configuration.pixelRatio + "px " + style.boardTextStyleWhiteXTablet.font, fill: style.boardTextStyleWhiteXTablet.fill};
      style.boardTextStyleWhiteO = {font: style.boardTextStyleWhiteOTablet.fontSize * configuration.pixelRatio + "px " + style.boardTextStyleWhiteOTablet.font, fill: style.boardTextStyleWhiteOTablet.fill};
    }else{
      style.boardTextStyleWhiteX = {font: style.boardTextStyleWhiteX.fontSize * configuration.pixelRatio + "px " + style.boardTextStyleWhiteX.font, fill: style.boardTextStyleWhiteX.fill};
      style.boardTextStyleWhiteO = {font: style.boardTextStyleWhiteO.fontSize * configuration.pixelRatio + "px " + style.boardTextStyleWhiteO.font, fill: style.boardTextStyleWhiteO.fill};
    }
  }

  /*
   * Listener to remind the user about the rules when he waits too long
   */
  function checkIdle() {
    var time = new Date().getTime();
    if (lastAction + configuration.showRulesDialogAfter < time && !remindedOfRules) {
      callbacks.dialogShowRules();
      lastAction = time;
      remindedOfRules = true;
    }
  }

  /*
   * Saves current state back to the app
   */
  function saveGame() {
    gameObject.savedGame = serializeBoard();
    gameObject.duration = Math.floor((new Date().getTime() - startTime.getTime()) / 1000);
    callbacks.saveGameItem(gameObject);
  }

  /*
   * Updates the timer (called each second once)
   */
  function updateTime() {
    if (!isFinished) {
      timerText.text = new Date(new Date().getTime() - startTime).toISOString().substr(14, 5);
      gameObject.duration = (new Date().getTime() - startTime.getTime()) / 1000;
    }
  }

  /*
   * Adds the timer on top of the board
   */
  function initTimer() {
    timerText = game.add.text(spacing.offsetLeftMobile, spacing.offsetHeaderMobile, new Date().getTime() - startTime, style.timerStyle);
  }

  /*
   * Adds either the rank or the level number on top of the board
   */
  function initRank() {
    if (gameObject.isCampaign) {
      rankText = game.add.text(spacing.offsetLeftMobile, timerText.y + spacing.spacingHeader, translations.rank.toUpperCase() + ' ' + gameObject.fieldSize + 'x' + gameObject.fieldSizeY, style.rankStyle);
      initProgress();
    } else {
      rankText = game.add.text(spacing.offsetLeftMobile, timerText.y + spacing.spacingHeader, translations.difficulty.toUpperCase() + ' ' + gameObject.fieldSize + 'x' + gameObject.fieldSizeY, style.rankStyle);
    }
  }

  /*
   * Adds the hint counter (with a click listener) on top of the board
   */
  function initHints() {
    hintsText = game.add.text(game.world.width - spacing.offsetLeftMobile - tileSize / 2, spacing.offsetHeaderMobile, translations.hints + "", style.hintsStyle);
    hintsText.anchor.set(0.5, 0);
    hintsLabel = game.add.text(game.world.width - spacing.offsetLeftMobile - tileSize / 2, spacing.offsetHeaderMobile + hintsText.height - 5, translations.hintsLabel.toUpperCase(), style.hintsLabelStyle);
    hintsLabel.anchor.set(0.5, 0);

    hintsText.inputEnabled = true;
    hintsText.events.onInputDown.add(function () {
      callbacks.dialogUseHint(hintApproved);
    }, this);
    hintsText.hitArea = new Phaser.Circle(0, 0, 80 * configuration.pixelRatio);
  }

  /*
   * Check if something wrong was set
   */
  function hintApproved(newHintCount, success, error) {
    var foundDifference = false;
    for (var column = 0; column < gameObject.fieldSize; column++) {
      for (var row = 0; row < gameObject.fieldSizeY; row++) {
        if (board[row][column] != null && board[row][column].blockType != boardSolution[row][column]) {
          if (boardRectangles[row][column].visible == false) {
            boardRectangles[row][column].visible = true;
            foundDifference = true;
          }
        }
      }
    }

    hintsText.text = newHintCount - 1;
    if (foundDifference) {
      success();
    } else {
      error();
    }
  }

  /*
   * Display current campaign process
   */
  function initProgress() {
    for (var i = 0; i < 5; i++) {
      var pic;
      if (i > translations.playerRank - 1) {
        pic = game.add.sprite(spacing.offsetLeftMobile + i * (12 + 4) * configuration.pixelRatio, rankText.y + spacing.spacingHeader, 'norank');
        pic.alpha = 0.7;
      } else {
        pic = game.add.sprite(spacing.offsetLeftMobile + i * (12 + 4) * configuration.pixelRatio, rankText.y + spacing.spacingHeader, 'rank');
      }
      pic.scale.set(32 / game.cache.getImage('rank').width);
    }
  }

  /*
   * Refreshes the current board (updates row labels etc.)
   */
  function checkBoard() {

    // Count blocks
    rowCounter = new Array(gameObject.fieldSize);
    columnCounter = new Array(gameObject.fieldSizeY);

    for (var i = 0; i < gameObject.fieldSize; i++) {
      columnCounter[i] = {X: 0, O: 0, u: 0};
    }

    for (var i = 0; i < gameObject.fieldSizeY; i++) {
      rowCounter[i] = {X: 0, O: 0, u: 0};
    }

    for (var x = 0; x < gameObject.fieldSize; x++) {
      for (var y = 0; y < gameObject.fieldSizeY; y++) {
        if (board[y] == undefined || board[y][x] == undefined) {
          columnCounter[x].u++;
          rowCounter[y].u++;
        } else if (board[y][x].blockType == blockTypes.O) {
          columnCounter[x].O++;
          rowCounter[y].O++;
        } else {
          columnCounter[x].X++;
          rowCounter[y].X++;
        }
      }
    }

    finishedCounter = 0;

    // Update labels at column & row
    for (var i = 0; i < gameObject.fieldSizeY; i++) {
      rowLabelsX[i].text = rowCounter[i].X;
      rowLabelsO[i].text = rowCounter[i].O;
      if (rowCounter[i].u == 0) {
        rowLabelsX[i].alpha = 0.5;
        rowLabelsO[i].alpha = 0.5;
        finishedCounter++;
      } else {
        rowLabelsX[i].alpha = 1;
        rowLabelsO[i].alpha = 1;
      }
    }

    for (var i = 0; i < gameObject.fieldSize; i++) {
      columnLabelsX[i].text = columnCounter[i].X;
      columnLabelsO[i].text = columnCounter[i].O;

      if (columnCounter[i].u == 0) {
        columnLabelsX[i].alpha = 0.5;
        columnLabelsO[i].alpha = 0.5;
        finishedCounter++;
      }
    }

    // Check if board is filled (no empty fields)
    gameObject.isFinished = finishedCounter == gameObject.fieldSize + gameObject.fieldSizeY;
    if (gameObject.isFinished) {
      gameObject.isCorrect = checkSolution();
      callbacks.gameListener(gameObject.isFinished, gameObject.isCorrect);
    }
  }

  /*
   * Validates if the current game board solution is correct
   */
  function checkSolution() {
    for (var row = 0; row < gameObject.fieldSize; row++) {
      for (var column = 0; column < gameObject.fieldSize; column++) {
        if (board[row][column].blockType != boardSolution[row][column]) {
          return false;
        }
      }
    }
    return true;
  }

  /*
   * Draws game board
   */
  function drawField() {

    // Draw background
    var bg = game.add.sprite(0, 0, 'background');
    bg.scale.set((Math.ceil(gameObject.fieldSize / 2 + 2) * tileSize + spacing.offsetTopMobile) / game.cache.getImage('background').height);

    // Draw field
    spacing.offsetLeftMobile += style.shadeBorderSize / 4;
    shadeBorder = game.add.graphics(spacing.offsetLeftMobile - style.shadeBorderSize / 2, spacing.offsetTopMobile - style.shadeBorderSize / 2);
    shadeBorder.beginFill(style.boardBackgroundColor, 1);
    shadeBorder.drawRect(0, 0, (gameObject.fieldSize + 1) * tileSize + style.shadeBorderSize / 2, (gameObject.fieldSizeY + 1) * tileSize + style.shadeBorderSize / 2);
    //shadeBorder.drawRect(0, 0, (gameObject.fieldSize + 1) * tileSize + 2 * style.shadeBorderSize, (gameObject.fieldSize / 2 + 2) * tileSize + 2 * style.shadeBorderSize);

    graphics = game.add.graphics(spacing.offsetLeftMobile, spacing.offsetTopMobile);
    boardRectangles = new Array(gameObject.fieldSize);

    for (var x = 0; x < gameObject.fieldSizeY; x++) {

      boardRectangles[x] = new Array(gameObject.fieldSizeY);

      // Set default style
      graphics.lineStyle(style.gameBorder, style.boardLineColor, style.gameBorder);
      graphics.beginFill(style.boardBackgroundColor);

      // Draw Grid squares
      for (var y = 0; y < gameObject.fieldSize; y++) {
        graphics.drawRect(y * tileSize, x * tileSize, tileSize, tileSize);

        rect = game.add.graphics(y * tileSize, x * tileSize);
        rect.beginFill(0xFF8A80);
        rect.drawRect(spacing.offsetLeftMobile + style.gameBorder / 2, spacing.offsetTopMobile + style.gameBorder / 2, tileSize - style.gameBorder, tileSize - style.gameBorder);
        rect.visible = false;
        boardRectangles[x][y] = rect;
      }

      // X Labels
      var rowX = game.add.text(spacing.offsetLeftMobile + 0.3 * tileSize + gameObject.fieldSize * tileSize, x * tileSize + spacing.offsetTopMobile + 0.3 * tileSize, x, style.boardTextStyleWhiteX);
      rowX.x = Math.round(rowX.x);
      rowX.y = Math.round(rowX.y);

      rowX.anchor.set(0.5);
      rowLabelsX[x] = rowX;

      // O Labels
      var rowO = game.add.text(spacing.offsetLeftMobile + 0.8 * tileSize + gameObject.fieldSize * tileSize, x * tileSize + spacing.offsetTopMobile + 0.7 * tileSize, x, style.boardTextStyleWhiteO);
      rowO.x = Math.round(rowO.x);
      rowO.y = Math.round(rowO.y);

      rowO.anchor.set(0.5);
      rowLabelsO[x] = rowO;

    }

    for (var y = 0; y < gameObject.fieldSize; y++) {
      columnsX = game.add.text(spacing.offsetLeftMobile + y * tileSize + 0.25 * tileSize, spacing.offsetTopMobile + gameObject.fieldSizeY * tileSize + 0.4 * tileSize, '', style.boardTextStyleWhiteX);
      columnsX.anchor.set(0.5);
      columnsX.x = Math.round(columnsX.x);
      columnsX.y = Math.round(columnsX.y);
      columnLabelsX[y] = columnsX;

      columnsO = game.add.text(spacing.offsetLeftMobile + y * tileSize + 0.75 * tileSize, spacing.offsetTopMobile + gameObject.fieldSizeY * tileSize + 0.75 * tileSize, '', style.boardTextStyleWhiteO);
      columnsO.anchor.set(0.5);
      columnsO.x = Math.round(columnsO.x);
      columnsO.y = Math.round(columnsO.y);
      columnLabelsO[y] = columnsO;
    }

    graphics.endFill();

    // Draw outer boarder
    graphics.lineStyle(style.outerGameBorder, style.boardLineColor, style.outerGameBorder);

    var diff = style.outerGameBorder - style.gameBorder;
    var dimension = gameObject.fieldSize * tileSize;
    var dimensionY = gameObject.fieldSizeY * tileSize;
    graphics.moveTo(-diff, -diff);
    graphics.lineTo(-diff, dimensionY + diff);
    graphics.lineTo(dimension + diff, dimensionY + diff);
    graphics.lineTo(dimension + diff, -diff);
    graphics.lineTo(-diff, -diff);
  }

  /*
   * Load csv and initialize arrays
   */
  function parseCSV() {
    lines = game.cache.getText('csv').split('\n');
    var header = lines[0].split(';');
    gameObject.fieldSize = parseInt(header[2]);
    gameObject.fieldSizeY = parseInt(header[3]);

    columnLabelsX = new Array(gameObject.fieldSizeY);
    columnLabelsO = new Array(gameObject.fieldSizeY);
    rowLabelsX = new Array(gameObject.fieldSize);
    rowLabelsO = new Array(gameObject.fieldSize);

    field = new Array(gameObject.fieldSize);
    for (var row = 0; row < gameObject.fieldSizeY; row++) {
      field[row] = new Array(gameObject.fieldSize);
    }
  }

  /*
   * Load current field to the field
   */
  function loadField() {
    board = undefined;
    board = new Array(gameObject.fieldSize);
    for (var x = 0; x < gameObject.fieldSizeY; x++) {
      board[x] = new Array(gameObject.fieldSize);
      for (var y = 0; y < gameObject.fieldSize; y++) {
        if (lines[x + 1][y] != '-') {
          addElement(x, y, lines[x + 1][y], preloadOpacity);
        }
      }
    }
  }

  /*
   * Load solution to the memory
   */
  function loadSolution() {
    boardSolution = new Array(gameObject.fieldSizeY);
    for (var x = 0; x < gameObject.fieldSizeY; x++) {
      boardSolution[x] = new Array(gameObject.fieldSize);
      for (var y = 0; y < gameObject.fieldSize; y++) {

        switch (lines[x + 1 + gameObject.fieldSizeY][y]) {
          case 'O':
            boardSolution[x][y] = 'O';
            break;
          case 'X':
            boardSolution[x][y] = 'X';
            break;
        }
      }
    }
  }

  /*
   * Tap listener (action that is executed when tapped)
   */
  function onTap(pointer) {
    // Update last action timer and close fab in case it was opened
    lastAction = new Date().getTime();
    callbacks.closeFab();

    // Calculate position on board
    var y = ((pointer.x - spacing.offsetLeftMobile) - ((pointer.x - spacing.offsetLeftMobile) % tileSize)) / tileSize;
    var x = ((pointer.y - spacing.offsetTopMobile) - ((pointer.y - spacing.offsetTopMobile) % tileSize)) / tileSize;

    if (x > -1 && x < gameObject.fieldSizeY && y > -1 && y < gameObject.fieldSize) {
      // Round coordinates to avoid issues on some devices due to lack of precision
      x = Math.round(x);
      y = Math.round(y);

      boardRectangles[x][y].visible = false;
      commandsStack.push({blockType: (board[x] && board[x][y] ? board[x][y].blockType : undefined), x: x, y: y});
      executeAction(x, y);
    }
  }

  /*
   * Execute next action at x y
   */
  function executeAction(x, y) {
    if (!isFinished && !(board[x] != undefined && board[x][y] != undefined && board[x][y].alpha == preloadOpacity)) {
      if (board[x] == undefined || board[x][y] == undefined) {
        addElement(x, y, blockTypes.X);
      } else if (board[x][y].blockType == blockTypes.X) {
        addElement(x, y, blockTypes.O);
      } else {
        clearField(x, y);
      }
    } else if (!isFinished && (board[x] != undefined && board[x][y] != undefined && board[x][y].alpha == preloadOpacity)) {
      callbacks.dialogNonRemovable();
    }
    checkBoard();
  }

  /*
   * Adds element of passed type at position x y (alpha is used to pass a preload opacity)
   */
  function addElement(x, y, blockType, alpha) {
    clearField(x, y);
    var pic = game.add.sprite(spacing.offsetLeftMobile + y * tileSize + style.gameBorder / 2, spacing.offsetTopMobile + x * tileSize + Math.ceil(style.gameBorder / 2), blockType);
    pic.scale.set((tileSize - style.gameBorder) / game.cache.getImage(blockType).width);
    if (alpha) pic.alpha = alpha;
    pic.blockType = blockType;
    board[x][y] = pic;
  }

  /*
   * Clears field at position x y
   */
  function clearField(x, y) {
    if (board[x] != undefined && board[x][y] != undefined) {
      if (board[x][y].alpha == preloadOpacity) {
        // Skip field because it was preloaded
        return false;
      }
      board[x][y].destroy();
      board[x][y] = null;
    }
    return true;
  }

  /*
   * Save current game to a passed game slot
   */
  function saveGameSlot(i) {
    gameObject.saveGameSlots[i] = {'board': serializeBoard(), 'time': new Date().getMilliseconds()};
    return gameObject;
  }

  /*
   * Restore and load game at the passed game slot
   */
  function restoreGameSlot(i, success, error) {
    if (gameObject.saveGameSlots && gameObject.saveGameSlots[i] && gameObject.saveGameSlots[i].board) {
      deserializeGame(gameObject.saveGameSlots[i].board, true);
      checkBoard()
      if (success) success(gameObject);
    } else {
      if (error) error();
    }
  }

  /*
   * Loads saved game state (including past game duration)
   */
  function loadSaveGame() {
    if (gameObject.duration > 0) {
      startTime = new Date(new Date().getTime() - (gameObject.duration * 1000));
    }
    deserializeGame(gameObject.savedGame);
    checkBoard();
  }

  /*
   * Clears all not preloaded elements
   */
  function clearAll() {
    for (var i = 0; i < gameObject.fieldSizeY; i++) {
      for (var j = 0; j < gameObject.fieldSize; j++) {
        clearField(i, j);
      }
    }
    checkBoard();
  }

  /*
   * Serialize current board state
   */
  function serializeBoard() {
    var saveBoard = new Array(board.length);
    for (var i = 0; i < board.length; i++) {
      saveBoard[i] = new Array(board.length);
      for (var j = 0; j < board.length; j++) {
        if (board[i] != undefined && board[i][j] != undefined && board[i][j].alpha != preloadOpacity) {
          saveBoard[i][j] = board[i][j].blockType;
        }
      }
    }
    return saveBoard;
  }

  /*
   * Deserializes saved game
   */
  function deserializeGame(savedBoard, force) {
    commandsStack = [];
    if (force) {
      clearAll();
    }

    if (savedBoard != undefined) {
      for (var i = 0; i < savedBoard.length; i++) {
        for (var j = 0; j < savedBoard.length; j++) {
          if (savedBoard[i] != undefined && savedBoard[i][j] != undefined && savedBoard[i][j] != undefined) {
            if (savedBoard[i][j] == 'X') {
              addElement(i, j, blockTypes.X);
            } else {
              addElement(i, j, blockTypes.O);
            }
          }
        }
      }
    }
  }

  /*
   * Undo previous step
   */
  function undo(success, error) {
    if (commandsStack.length > 0) {
      var cmd = commandsStack.pop();

      if (cmd.blockType == undefined) {
        clearField(cmd.x, cmd.y);
      } else {
        addElement(cmd.x, cmd.y, cmd.blockType);
      }
      checkBoard();
      if (success) success();
    } else {
      if (error) error();
    }
  }

  game.state.add("PlayGame", playGame);
  game.state.start("PlayGame");

  return {
    'saveGameSlot': saveGameSlot,
    'restoreGameSlot': restoreGameSlot,
    'hintApproved': hintApproved,
    'saveGame': saveGame,
    'clearAll': clearAll,
    'undo': undo
  }
};
