/**
 * Abstraction service to use Google Analytics
 * @author Lukas Schönbächler
 * @see https://github.com/danwilson/google-analytics-plugin
 */
angular.module('app.services')
    .factory('Analytics', function ($ionicPlatform, $log, ANALYTICS_ID) {

        var isReady = false;

        $ionicPlatform.ready(function () {
            if(window.ga) {
                try {
                    // Init tracker
                    window.ga.startTrackerWithId(ANALYTICS_ID);

                    // To enabling Advertising Features in Google Analytics allows you to take advantage of Remarketing, Demographics & Interests reports, and more
                    window.ga.setAllowIDFACollection(true);
                    isReady = true;

                    // Set app version
                    cordova.getAppVersion.getVersionCode().then(function (version) {
                        window.ga.setAppVersion(version);
                    });
                    $log.debug('Analytics: Active');
                } catch (e) {
                    $log.warn('Analytics: Not active');
                    $log.error(e);

                }
            }else{
                $log.debug('Analytics: Not active - dev mode');
            }
        });

        var Analytics = {

            /**
             * Tracks a view
             */
            trackView: function (title) {
                if (isReady) {
                    window.ga.trackView(title);
                    $log.debug('Analytics: trackView title:' + title);
                } else {
                    $log.warn('Analytics: View not tracked because not ready yet');
                }
            },

            /**
             * Tracks an event
             */
            trackEvent: function (category, action, label, value) {
                if (isReady) {
                    window.ga.trackEvent(category, action, label, value)
                    $log.debug('Analytics: trackEvent category:' + category + ' action:' + action + ' label:' + label + ' value:' + value);
                } else {
                    $log.warn('Analytics: Event not tracked because not ready yet');
                }
            },

            /**
             * Tracks the game time
             * @param {number} id       Id of the game
             * @param {number} value    Duration in seconds
             */
            trackGameTime: function (id, value) {
                Analytics.trackEvent('GAME', 'TIME', id, value);
                $log.debug('Analytics: trackGameTime id:' + id + ' + value:' + value);
            },

            /**
             * Tracks the game result
             * @param {number} id        Id of the game
             * @param {boolean} value    Indicator if game was correct
             */
            trackGameResult: function (id, value) {
                Analytics.trackEvent('GAME', 'FINISHED', id, value);
                $log.debug('Analytics: trackGameResult id:' + id + ' + value:' + value);
            },

            /**
             * Tracks an exception
             * @param {string} description  Error description
             * @param {boolean} fatal       Indicates if the crash was fatal
             */
            trackException: function (description, fatal) {
                if (isReady) {
                    window.ga.trackException(description, fatal)
                    $log.debug('Analytics: trackException description:' + description + " + fatal:" + fatal);
                } else {
                    $log.warn('Analytics: Exception not tracked because not ready yet');
                }
            }
        };
        return Analytics;
    });
