angular.module('app.services')
  .factory('GameHelper', function ($translate, $ionicPopup, $state, ItemDB, Profile, Utils, Store) {

    var noHints = function () {
      Profile.consumeHint();
      Utils.showTranslatedMessage('NO_HINTS');
    };

    var consumeHint = function(){
      Profile.consumeHint();
    };

    var GameHelper = {

      saveGameItem: function (item) {
        ItemDB.insertOrUpdate(item);
        if (!item.isFinished) {
          Profile.setResumableGameId(item.id);
        }
      },

      dialogNonRemovable: function () {
        $translate('NONREMOVABLE').then(function (translation) {
          if (window.plugins) {
            window.plugins.toast.showLongBottom(translation);
          } else {
            console.log(translation);
          }
        });
      },


      dialogShowRules: function () {
        if (localStorage.getItem('noHelp')) return;
        // Show Rules
        $translate(['RULES_REMINDER_TITLE', 'RULES_REMINDER_DESCRIPTION', 'OK', 'NEVER', 'CANCEL']).then(function (translations) {
          var confirmPopup = $ionicPopup.show({
            title: translations['RULES_REMINDER_TITLE'],
            template: translations['RULES_REMINDER_DESCRIPTION'],
            cssClass: "popup-vertical-buttons",
            buttons: [
              {
                text: translations['OK'],
                type: 'button-energized',
                onTap: function (e) {
                  $state.go('tutorial');
                }
              },
              {
                text: translations['NEVER'],
                type: 'button-assertive',
                onTap: function (e) {
                  localStorage.setItem('binoxxo.noHelp', true);
                }
              },
              {
                text: translations['CANCEL']
              }
            ]
          });
        });
      },

      dialogUseHint : function (callback) {
        $translate(['OK', 'CANCEL', 'HINTS_CONFIRM']).then(function (translations) {
          $ionicPopup.show({
            template: translations.HINTS_CONFIRM,
            buttons: [
              {text: translations.CANCEL},
              {
                text: translations.OK,
                type: 'button-positive',
                onTap: function () {
                  if (Profile.getHints() > 0) {
                    callback(Profile.getHints(), consumeHint, noHints);
                  } else {
                    if (ionic.Platform.isIOS()) {
                      $translate(['PURCHASE', 'CANCEL', 'STORE_HINTS_DESCRIPTION']).then(function (translations) {
                        $ionicPopup.show({
                          template: translations.STORE_HINTS_DESCRIPTION,
                          buttons: [
                            {text: translations.CANCEL},
                            {
                              text: translations.PURCHASE,
                              type: 'button-positive',
                              onTap: function () {
                                ignorePause = true;
                                Store.purchaseHints(function () {
                                  ignorePause = false;
                                  callback(Profile.getHints(), consumeHint, noHints);
                                }, function () {
                                  ignorePause = false;
                                });
                              }
                            }
                          ]
                        });
                      });
                    } else {
                      ignorePause = true;
                      Store.purchaseHints(function () {
                        console.log('purchaseHints');
                        ignorePause = false;
                        callback(Profile.getHints(), consumeHint, noHints);
                      }, function () {
                        ignorePause = false;
                      });
                    }
                  }
                }
              }
            ]
          });
        });
      }

    };
    return GameHelper;
  });
