angular.module('app.services')
  .factory('ItemDB', function (Storage, PACKAGES, ITEMS, CAMPAIGN_THEMES, DIFFICULTIES_THEMES, STORAGE_PREFIX, TUTORIAL_THEME, PackageDB) {

    var cachedItems = {};
    var cachedItemsList = [];
    var savedItems = {};

    var addObjects = function (list, extendWith) {
      var newExtendWith = angular.extend(extendWith, {duration: 0, state: 0, score: 0, saveGameSlots: []});
      for (var i = 0; i < list.length; i++) {
        var withoutExtension = list[i].slice(0, -4);
        //if (cachedItems.filter(byId)[0] == undefined) {
        if (cachedItems[withoutExtension] == undefined) {
          ItemDB.insertOrUpdate(angular.extend({id: withoutExtension.toUpperCase()}, newExtendWith), true);
        }
      }
    };

    var rebuildList = function () {
      cachedItemsList = [];
      for (var key in cachedItems) {
        if (cachedItems.hasOwnProperty(key)) {
          cachedItemsList.push(cachedItems[key]);
        }
      }
    };

    var ItemDB = {
      // Wenn rebuild
      selectAll: function () {
        return cachedItemsList;
      },

      /*
       selectAllFreeOrPurchased: function () {
       var items = [];
       for (var i = 0; i < cachedItemsList.length; i++) {
       if (cachedItemsList[i].purchased) {
       items.push(cachedItemsList[i]);
       }
       }
       return items;
       },
       */

      selectById: function (id) {
        return cachedItems[id];
      },

      insertOrUpdate: function (item, noSave) {
        cachedItems[item.id] = item;
        if (!noSave) {
          savedItems[item.id] = item;
          Storage.setItem('savedItems', savedItems);
          rebuildList();
        }
      },

      count: function () {
        return cachedItemsList.length;
      }
    };


    var checkData = function () {

      var difficulties = ["easy", "middle", "hard"];
      var purchasedPackages = PackageDB.selectAllPurchased();


      for (var j = 0; j < difficulties.length; j++) {
        for (var i = 0; i < PACKAGES.length; i++) {
          if (PACKAGES[i].category == difficulties[j]) {
            if (ITEMS[difficulties[j]][PACKAGES[i].id])

            /*
             var purchased = false;
             for (var x = 0; x < purchasedPackages.length; x++) {
             if (PACKAGES[i].id == purchasedPackages[x].id) {
             purchased = true;
             break;
             }
             }
             */

              addObjects(Object.keys(ITEMS[difficulties[j]][PACKAGES[i].id]), {
                path: 'ITEMS/' + [difficulties[j]] + "/" + PACKAGES[i].id + "/",
                theme: DIFFICULTIES_THEMES[j],
                category: difficulties[j],
                //purchased: i < 3 || purchased,
                isCampaign: false,
                packageId: PACKAGES[i].id
              });
          }
        }
      }

      var promoKeys = Object.keys(ITEMS['promo'])
      for (var i = 0; i < promoKeys.length; i++) {
        var promoPackageKeys = Object.keys(ITEMS['promo'][promoKeys[i]])
        for(var j = 0; j < promoPackageKeys.length; j++){
          var item = ITEMS['promo'][promoKeys[i]][promoPackageKeys[j]]
          var withoutExtension = item.substr(0, item.length - 4)
          var id = withoutExtension.substr(withoutExtension.lastIndexOf('/') + 1, withoutExtension.length)
          ItemDB.insertOrUpdate({
            id: id,
            path: 'ITEMS/promo/' + promoKeys[i] + '/' ,
            theme: 'cyan',
            isCampaign: false,
            category: 'easy',
            duration: 0,
            state: 0,
            score: 0
          }, true);
        }
      }


      var keys = Object.keys(ITEMS.campaign);

      var steps = 5;
      for (var j = 0; j < CAMPAIGN_THEMES.length; j++) {
        addObjects(keys.slice(j * steps, (j + 1) * steps), {
          path: "ITEMS/campaign/",
          category: 'easy',
          isCampaign: true,
          theme: CAMPAIGN_THEMES[j]
        });
      }

      ItemDB.insertOrUpdate({
        id: 'tutorial',
        path: 'ITEMS/tutorial/',
        theme: TUTORIAL_THEME,
        isCampaign: false,
        category: 'easy',
        duration: 0,
        state: 0,
        score: 0
      }, true);

      var oldSavedItems = JSON.parse(localStorage.getItem(STORAGE_PREFIX + 'tmp_itemData'));
      if (oldSavedItems) {
        console.warn('Do item migration');
        for (var i = 0; i < oldSavedItems.length; i++) {
          if (oldSavedItems[i].savedGame != undefined) {
            cachedItems[oldSavedItems[i].id] = oldSavedItems[i];
            savedItems[oldSavedItems[i].id] = oldSavedItems[i];
          }
        }
        localStorage.removeItem(STORAGE_PREFIX + 'tmp_itemData');
      }

      Storage.getItem('savedItems', function (value) {
        savedItems = angular.extend(savedItems, value);
        // restore saved data
        for (var key in savedItems) {
          if (savedItems.hasOwnProperty(key) && cachedItems[savedItems[key].id] != undefined) {
            cachedItems[key] = savedItems[key];
          }
        }
        if (oldSavedItems) Storage.setItem('savedItems', savedItems);
        rebuildList();
      }, function(error){
        if (oldSavedItems) Storage.setItem('savedItems', savedItems);
        rebuildList();
      });


      rebuildList();
    };

    checkData();


    return ItemDB;
  });
