angular.module('app.services')
    .factory('PackageDB', function ($http, $log, $rootScope, $filter, Storage, PACKAGES, LEADERBOARDS, STORAGE_PREFIX) {

        var CONST_PACKAGEDATA = "packageData";

        var allPackages = PACKAGES;
        var purchasedPackages = {};


        var PackageDB = {
            selectAll: function () {
                return allPackages;
            },

            selectAllNotPurchased: function () {
                var packages = [];
                for (var i = 0; i < allPackages.length; i++) {
                    if (!allPackages[i].purchased) {
                        packages.push(allPackages[i]);
                    }
                }
                return packages;
            },

            selectAllPurchased: function () {
                var packages = [];
                for (var i = 0; i < allPackages.length; i++) {
                    if (allPackages[i].purchased) {
                        packages.push(allPackages[i]);
                    }
                }
                return packages;
            },

            selectById: function (id) {
                for (var i = 0; i < allPackages.length; i++) {
                    if (allPackages[i].id == id) {
                        return allPackages[i];
                    }
                }
                return null;
            },

            insert: function (pack, success, noSave) {
                //allPackages.push(pack);
                purchasedPackages[pack.id] = true;
                if (!noSave) {
                    Storage.setItem("purchasedPackages", purchasedPackages, success);
                } else {
                    if (success) success();
                }
            },

            insertOrUpdate: function (pack, success, noSave) {

                if (!PackageDB.update(pack, success, noSave)) {
                    PackageDB.insert(pack, success, noSave);
                    return true;
                }
                else {
                    return false;
                }
            },

            update: function (pack, success, noSave) {
                for (var i = 0; i <= allPackages.length; i++) {
                    try {
                        if (allPackages[i].id == pack.id) {
                          purchasedPackages[pack.id] = true;
                          if (!noSave){
                            Storage.setItem("purchasedPackages", purchasedPackages, success);
                          }
                            return true;
                        }
                    } catch (e) {
                        $log.error(allPackages);
                    }
                }
                return false;
            },

            delete: function (pack) {
                for (var i = 0; i <= allPackages.length; i++) {
                    if (allPackages[i].id == pack.id) {
                        allPackages.splice(i, 1);
                        break;
                    }
                }
            },

            // Get number of notes.
            count: function () {
                return allPackages.length;
            }
        };

        // Migration
        var oldPackages = JSON.parse(localStorage.getItem(STORAGE_PREFIX + 'tmp_packageData'));
        if (oldPackages) {
          console.warn('Do package migration');
          for (var i = 0; i < oldPackages.length; i++) {
            if (oldPackages[i].purchased) {
              purchasedPackages[oldPackages[i].id] = true;
            }
          }

          for (var i = 0; i < allPackages.length; i++) {
            if (!allPackages[i].purchased) allPackages[i].purchased = purchasedPackages[allPackages[i].id];
          }

          localStorage.removeItem(STORAGE_PREFIX + 'tmp_packageData');
        }

        Storage.getItem('purchasedPackages', function (value) {
          purchasedPackages = angular.extend(purchasedPackages, value);

          for (var i = 0; i < allPackages.length; i++) {
            if (!allPackages[i].purchased) allPackages[i].purchased = purchasedPackages[allPackages[i].id];
          }
          if (oldPackages) Storage.setItem('purchasedPackages', purchasedPackages);

        }, function(error){
          console.log('error', purchasedPackages);
          if (oldPackages) Storage.setItem('purchasedPackages', purchasedPackages);

        });

      return PackageDB;
    });
