angular.module('app.services')
  .factory('Profile', function ($q, $log, $rootScope, $filter, $state, $ionicPopup, $translate, Storage,Gameservice, LEADERBOARDS, ACHIEVEMENTS, STORAGE_PREFIX) {
    var levelProRank = 5;
    var maxRank = 5;
    var CONST_PROFILE = "profile";

    var profile = {level: 1, currentTab:0, campaignTime: 0, campaignFinished: false, hints:0, hasPurchases: false, function_save_load: false, startCounter: 1, hideObject: {}};
    var tmpProfile = localStorage.getItem(STORAGE_PREFIX + "tmp_" + CONST_PROFILE);
    if (tmpProfile) {
      profile = JSON.parse(tmpProfile);
    }

    Storage.getItem(CONST_PROFILE, function(p){
      profile = p;
    });

    /*
    $rootScope.$on("migrate_profile",function () {
      $log.debug("Profile: Broadcast received");
      Storage.getItem(CONST_PROFILE, function(p){
        profile = p;
      });
    });

    */

    if(profile.startCounter == 1){
      $translate(['WELCOME_TITLE', 'WELCOME_MESSAGE', 'SKIP', 'RULES', 'VIDEO']).then(function (translations) {
        $ionicPopup.confirm({
          title: translations.WELCOME_TITLE,
          template: translations.WELCOME_MESSAGE,
          cssClass: "popup-vertical-buttons",
          buttons: [
            {
              text: translations.RULES,
              onTap: function(e) {
                $state.go('tutorial');
              }
            },
            {
              text: translations.VIDEO,
              onTap: function(e) {
                $state.go('video');
              }
            },
            {
              text: translations.SKIP
            }]
        });
      });
    }

    var getLeaderboardByLabel = function(label){
      return $filter('filter')(LEADERBOARDS, {label: label.toUpperCase()})[0];
    };

    var storeScore = function (leaderboard, score) {
      Storage.getItem(leaderboard.label + '_score', function(oldScore){
        Storage.setItem(leaderboard.label + '_score', oldScore + score);
        Gameservice.addPoints(leaderboard, oldScore + score);
      }, function(){
        Storage.setItem(leaderboard.label + '_score', score);
        Gameservice.addPoints(leaderboard, score);
      });
    };


    var Profile = {
      /*
       *  Calculate & optinally stores points for a given duration and leaderboard
       * @param {int}         Game duration in seconds
       * @param {Boolean}     Indicator if result should be saved to the users profile
       * @param {Leaderboard} Optional, if not passed, the CAMPAIGN Leaderboard will be used
       */
      calculatePoints: function (duration, storeResult, leaderboardLabel) {
        if(leaderboardLabel == undefined){
          var pointsCampaign = parseInt((600 * this.getRank()) - duration);
          if (pointsCampaign > 0) {
            if(storeResult){
              storeScore(getLeaderboardByLabel("CAMPAIGN"), pointsCampaign);
            }
            return pointsCampaign;
          } else {
            return 0;
          }
        }else{
          var leaderboard = getLeaderboardByLabel(leaderboardLabel);
          var points = parseInt(leaderboard.initialpoints - duration);
          if(points > 0){
            if(storeResult){
              storeScore(leaderboard, points);
              storeScore(getLeaderboardByLabel("OVERALL"), points);
            }
            return points;
          }else{
            return 0;
          }
        }
      },

      getHasFunction: function(key){
        if(profile[key]){
          return profile[key];
        }
        return false;
      },

      setHasFunction: function(key){
        profile[key] = true;
        this.setProfile(profile);
      },


      getHasNextCampaign: function(){
        var level = Profile.getLevel() + 1;
        return level <= 25;
      },

      getRank: function () {
        var rank = Math.ceil(this.getLevel() / levelProRank);
        if(rank > maxRank){
          return maxRank;
        }
        return rank;
      },

      getRankLabel: function () {
        return ACHIEVEMENTS[Profile.getRank() - 1].label;
      },

      getNextRank: function () {
        var rank = Profile.getRank() + 1;
        return (rank > maxRank )? undefined : rank;
      },

      getNextRankLabel: function () {
        var nextRank = this.getNextRank() - 1;
        if(nextRank){
          return ACHIEVEMENTS[nextRank].label;
        }
        return undefined;
      },

      getProfile: function () {
        return profile;
      },

      getHints: function () {
        return this.getProfile().hints;
      },

      setProfile: function (newProfile) {
        profile = newProfile;
        Storage.setItem(CONST_PROFILE, profile, function(){
          $log.info("Profile: Profile updated");
        });
        localStorage.setItem(STORAGE_PREFIX + "tmp_profile", JSON.stringify(profile));
        return profile;
      },

      setResumableGameId: function(id){
        profile.resumeableGameId = id;
        this.setProfile(profile);
      },

      removeResumableGameId: function(){
        delete profile.resumeableGameId;
        this.setProfile(profile);
      },

      getResumableGameId: function(){
        return profile.resumeableGameId;
      },

      increaseStartcounter: function (){
        profile.startCounter++;
        this.setProfile(profile);
      },

      addHints: function (){
        profile.hints += 10;
        this.setProfile(profile);
        this.setHasPurchases();
      },

      consumeHint: function (){
        profile.hints--;
        this.setProfile(profile);
      },

      setHasPurchases: function(){
        profile.hasPurchases = true;
        return this.setProfile(profile);
      },

      hasPurchases: function () {
        return profile.hasPurchases;
      },

      getLevel: function () {
        return profile.level;
      },

      getCurrentTab: function () {
        return profile.currentTab;
      },

      setCurrentTab: function(i){
        profile.currentTab = i;
        this.setProfile(profile);
      },

      getLevelInRank: function(){
        return this.getLevel() - (levelProRank * (this.getRank() - 1));
      },

      increaseLevel: function () {
        if(profile.level != levelProRank * maxRank) {
          profile.level++;
          Profile.setProfile(profile);
        }else{
          profile.campaignFinished = true;
          Profile.setProfile(profile);
        }
        Gameservice.unlockAchievement(ACHIEVEMENTS[this.getRank() - 1].id);
      },

      addCampaignTime: function (durationInSeconds) {
        profile.campaignTime += durationInSeconds;
        this.setProfile(profile);
      },

      toggleHeader: function(difficulty, pack){
        if(profile.hideObject[difficulty]){
          if(profile.hideObject[difficulty][pack] !== undefined){
            profile.hideObject[difficulty][pack] = !profile.hideObject[difficulty][pack];
          }else{
            profile.hideObject[difficulty][pack] = true;
          }
        }else{
          profile.hideObject[difficulty] = {};
          profile.hideObject[difficulty][pack] = true;
        }
        this.setProfile(profile);
        return profile.hideObject;
      }


    };
    return Profile;
  });
