/**
 * Abstraction of local storage service
 * @author Lukas Schönbächler
 */
angular.module('app.services')
    .factory('Storage', function ($log, $rootScope, $ionicPlatform, Analytics, STORAGE_PREFIX) {

        useNative = false;
        if (window.NativeStorage) {
            useNative = true;
            $log.debug("Storage: Use nativestorage");
        } else {
            $log.warn("Storage: Use localstorage - dev mode");
        }

        isMigrationCheckDone = false;
        isMigrationRequired = false;
        migrationCheckListeners = [];
        var notifyListeners = function(){
            $log.warn("Storage: isMigrationRequired=" + isMigrationRequired);
            if(isMigrationRequired) {
                for (var i = 0; i < migrationCheckListeners.length; i++) {
                    migrationCheckListeners[i]();
                }
            }
        };

        $ionicPlatform.ready(function () {
            try{
                cordova.getAppVersion.getVersionCode().then(function (version) {
                    Storage.getItem('dataVersion', function (value) {
                        isMigrationRequired = value != version;
                        isMigrationCheckDone = true;
                        notifyListeners();
                        Storage.setItem('dataVersion', version);
                    }, function () {
                        isMigrationRequired = true;
                        isMigrationCheckDone = true;
                        notifyListeners();
                        Storage.setItem('dataVersion', version);
                    });
                });
            }catch(e){
                // Emulate version
                // isMigrationRequired = Math.random() >= 0.5;
                isMigrationCheckDone = true;
                notifyListeners();
            }
        });


        var Storage = {

            isDataMigrationRequired: function(ifRequired){
                if(isMigrationCheckDone && isMigrationRequired) return ifRequired();
                migrationCheckListeners.push(ifRequired);
            },

            setItem: function (key, value, callback) {
                key = STORAGE_PREFIX + key;
                if (useNative) {
                    NativeStorage.setItem(key, value, function (value) {
                        $log.debug("Storage: setItem('" + key + "', '" + value + "')");
                        $log.debug(value);
                        if (callback) callback(true, key, value);
                    }, function (error) {
                        $log.error("Storage: Error setItem('" + key + "', '" + value + "')");
                        Analytics.trackException(error, false);
                        if (callback) callback(error);
                    });
                } else {
                    value = JSON.stringify(value);
                    var worked = localStorage.setItem(key, value);
                    $log.debug("Storage: setItem('" + key + "', '" + value + "') => " + worked);
                    if (callback) callback(worked, key, value);
                }
            },

            getItem: function (key, success, error) {
                key = STORAGE_PREFIX + key;
                if (useNative) {
                    NativeStorage.getItem(key, function (value) {
                        if (value == null) {
                            $log.warn("Storage: Error getItem('" + key + "')");
                            if (error) error("Item " + key + " does not exist");
                        } else {
                            $log.debug("Storage: getItem('" + key + "') => '" + value + "'");
                            if (success) success(value);
                        }
                    }, function (e) {
                        if(e.code == -2){
                            $log.warn("Storage: Error getItem('" + key + "')");
                            if (error) error("Item " + key + " does not exist");
                        }else{
                            $log.error("Storage: Error getItem('" + key + "')");
                            Analytics.trackException(e, false);
                            if (error) error(e);
                        }
                    });
                } else {
                    var value = JSON.parse(localStorage.getItem(key));
                    if (value == null) {
                        $log.warn("Storage: Error getItem('" + key + "')");
                        if (error) error("Item " + key + " does not exist");
                    } else {
                        $log.debug("Storage: getItem('" + key + "') => '" + value + "'");
                        if (success) success(value);
                    }
                }
            },

            remove: function (key, success, error) {
                key = STORAGE_PREFIX + key;
                if (useNative) {
                    NativeStorage.remove(key, function (value) {
                        $log.debug("Storage: remove('" + key + "')");
                        if (success) success(key);
                    }, function (e) {
                        $log.error("Storage: Error remove('" + key + "')");
                        Analytics.trackException(e, false);
                        if (error) error(e);
                    });
                } else {
                    localStorage.removeItem(key);
                    $log.debug("Storage: remove('" + key + "')");
                    if (success) success(key);
                }
            },

            clear: function (callback) {
                if (useNative) {
                    NativeStorage.clear(function () {
                        $log.debug("Storage: clear()");
                        if (callback) callback(null);
                    }, function (error) {
                        $log.error("Storage: clear " + error);
                        if (callback) callback(err);
                    });
                } else {
                    localStorage.clear();
                    $log.debug("Storage: clear()");
                    if (callback) callback(null);
                }
            }
        };


        return Storage;
    });
